#include <QDebug>
#include <QMutex>
#include <QtCore/QCoreApplication>
#include "ftpcleaner.h"

FtpCleaner::FtpCleaner(QObject* parent,QString host,int port,QString folder,
                       QString user,QString password,int nbFiles2Keep)
         : QObject(parent)
 {
    ftp=NULL;
    this->host = host;
    this->port = port;
    this->folder = folder;
    this->user = user;
    this->passwd = password;
    this->nbFiles2Keep = nbFiles2Keep;
    this->encounteredError = false;
 }

 void FtpCleaner::ftpCommandFinished(int commandId, bool error)
 {
     if (error)
         encounteredError = true;

     if (!error && ftp->currentCommand() == QFtp::ConnectToHost)
     {
        qDebug() << "connecting...DONE";
        qDebug() << "logging in... : " << user << "@" << host << ":" << port << "/" << folder;
        ftp->login(user,passwd);
     }
     else if (!error && ftp->currentCommand() == QFtp::Login)
     {
        qDebug() << "logging in...DONE";
        qDebug() << "changing dir...";
        ftp->cd(folder);
     }
     else if (!error && ftp->currentCommand() == QFtp::Cd)
     {
        qDebug() << "changing dir...DONE";
        qDebug() << "retrieving filelist...";
        fileList.clear();
        ftp->list("."); //we already switched to the correct folder
     }
     else if (!error && ftp->currentCommand() == QFtp::List)
     {
        qDebug() << "retrieving filelist...DONE";
        qSort(fileList);
        for (int i = 0; i < fileList.length(); ++i)
        {
            if (i < fileList.length() - nbFiles2Keep)
            {
                qDebug() << "DELE: " << fileList[i].name();
                ftp->remove(fileList[i].name());
            }
            else
            {
                qDebug() << "SKIP: " << fileList[i].name();
            }
        }
        ftp->close();
     }
     else if (ftp->currentCommand() == QFtp::Remove)
     {
         if (error)
         {
           qDebug() << "Error removing file: " << ftp->errorString();
           qDebug() << "disconnecting...";
           ftp->close();
         }
     }
     else if (ftp->currentCommand() == QFtp::Close)
     {
        qDebug() << "Disconnecting...DONE";
        qDebug() << "quiting...";
        if (encounteredError)
        {
            qApp->exit(1);
        }
        else
        {
            qApp->exit();
        }
     }
     else
     {
        if (error)
        {
            qDebug() << "Error: " << ftp->errorString();
        }
        else
        {
            qDebug() << "Dunno what to do...";
        }
        qDebug() << "disconnecting...";
        ftp->close();
     }
 }

 void FtpCleaner::addToFileList(const QUrlInfo& info)
 {
     SortedUrlInfo file(info);
     if (info.isFile())
        fileList.append(file);
 }

 void FtpCleaner::startCleanup()
 {
     ftp = new QFtp(this);
     connect(ftp,SIGNAL(commandFinished(int,bool)),this,SLOT(ftpCommandFinished(int,bool)));
     connect(ftp,SIGNAL(listInfo(QUrlInfo)),this,SLOT(addToFileList(QUrlInfo)));

     qDebug() << "connecting...";
     ftp->connectToHost(host,port);
 }
